@@ -113,6 +113,13 @@ use clap::{value_parser, Arg, ArgAction, Command};
 #[allow(unused_imports)]
 use tracing::{error, info, warn};
 
+#[cfg(all(
+    feature = "onion-service-client",
+    feature = "experimental-api",
+    feature = "keymgr"
+))]
+use clap::Subcommand as _;
+
 /// Shorthand for a boxed and pinned Future.
 type PinnedFuture<T> = std::pin::Pin<Box<dyn futures::Future<Output = T>>>;
 
@@ -196,7 +203,7 @@ async fn run<R: Runtime>(
                 .make_secure_dir(parent)?;
             // It's just a unix thing; if we leave this sitting around, binding to it won't
             // work right.  There is probably a better solution.
-            if path.exists() {
+            if path.try_exists()? {
                 std::fs::remove_file(&path)?;
             }
 
@@ -460,6 +467,12 @@ where
         }
     }
 
+    cfg_if::cfg_if! {
+        if #[cfg(all(feature = "onion-service-client", feature = "experimental-api", feature = "keymgr"))] {
+            let clap_app = subcommands::hsc::HscSubcommands::augment_subcommands(clap_app);
+        }
+    }
+
     // Relay subcommand
     cfg_if::cfg_if! {
         if #[cfg(feature = "relay")] {
@@ -598,6 +611,15 @@ where
         }
     }
 
+    // Check for the optional "hsc" subcommand.
+    cfg_if::cfg_if! {
+        if #[cfg(all(feature = "onion-service-client", feature = "experimental-api", feature = "keymgr"))] {
+            if let Some(hsc_matches) = matches.subcommand_matches("hsc") {
+                return subcommands::hsc::run(runtime, hsc_matches, &client_config);
+            }
+        }
+    }
+
     // Check for the optional "relay" subcommand.
     cfg_if::cfg_if! {
         if #[cfg(feature = "relay")] {
