@@ -94,8 +94,8 @@ impl ExitPolicy {
         // tor-netdir or tor-relay-selection.  That way we wouldn't need to
         // invoke these Relay-specific methods in tor-circmgr.
         Self {
-            v4: relay.ipv4_policy(),
-            v6: relay.ipv6_policy(),
+            v4: relay.low_level_details().ipv4_policy(),
+            v6: relay.low_level_details().ipv6_policy(),
         }
     }
 
@@ -219,7 +219,7 @@ impl TargetCircUsage {
         rng: &mut R,
         netdir: crate::DirInfo<'a>,
         guards: Option<&GuardMgr<RT>>,
-        #[cfg(all(feature = "vanguards", feature = "hs-common"))] vanguards: &VanguardMgr,
+        #[cfg(all(feature = "vanguards", feature = "hs-common"))] vanguards: &VanguardMgr<RT>,
         config: &crate::PathConfig,
         now: SystemTime,
     ) -> Result<(
@@ -536,6 +536,7 @@ pub(crate) mod test {
     use tor_netdir::testnet;
     #[cfg(all(feature = "vanguards", feature = "hs-common"))]
     use tor_persist::TestingStateMgr;
+    use tor_rtmock::MockRuntime;
 
     impl IsolationTokenEq for TargetCircUsage {
         fn isol_eq(&self, other: &Self) -> bool {
@@ -652,9 +653,9 @@ pub(crate) mod test {
         assert!(!ep_bad.allows_port(TargetPort::ipv6(80)));
 
         // Check is_supported_by while we're here.
-        assert!(TargetPort::ipv4(80).is_supported_by(&web_exit));
-        assert!(!TargetPort::ipv6(80).is_supported_by(&web_exit));
-        assert!(!TargetPort::ipv6(80).is_supported_by(&bad_exit));
+        assert!(TargetPort::ipv4(80).is_supported_by(&web_exit.low_level_details()));
+        assert!(!TargetPort::ipv6(80).is_supported_by(&web_exit.low_level_details()));
+        assert!(!TargetPort::ipv6(80).is_supported_by(&bad_exit.low_level_details()));
     }
 
     #[test]
@@ -870,7 +871,7 @@ pub(crate) mod test {
         let netdir = testnet::construct_netdir().unwrap_if_sufficient().unwrap();
         let di = (&netdir).into();
         let config = crate::PathConfig::default();
-        let guards: OptDummyGuardMgr<'_> = None;
+        let guards: OptDummyGuardMgr<'_, MockRuntime> = None;
         let now = SystemTime::now();
 
         // Only doing basic tests for now.  We'll test the path
@@ -878,7 +879,13 @@ pub(crate) mod test {
         // and friends.
 
         #[cfg(all(feature = "vanguards", feature = "hs-common"))]
-        let vanguards = VanguardMgr::new(&Default::default(), TestingStateMgr::default()).unwrap();
+        let vanguards = VanguardMgr::new(
+            &Default::default(),
+            MockRuntime::new(),
+            TestingStateMgr::default(),
+            false,
+        )
+        .unwrap();
 
         // First, a one-hop directory circuit
         let (p_dir, u_dir, _, _) = TargetCircUsage::Dir
@@ -953,7 +960,7 @@ pub(crate) mod test {
         // We'll always get exits for these, since we try to build
         // paths with an exit if there are any exits.
         assert!(policy.allows_some_port());
-        assert!(last_relay.policies_allow_some_port());
+        assert!(last_relay.low_level_details().policies_allow_some_port());
         assert_isoleq!(
             usage,
             SupportedCircUsage::Exit {
@@ -978,11 +985,17 @@ pub(crate) mod test {
         .unwrap();
         let di = (&netdir).into();
         let config = crate::PathConfig::default();
-        let guards: OptDummyGuardMgr<'_> = None;
+        let guards: OptDummyGuardMgr<'_, MockRuntime> = None;
         let now = SystemTime::now();
 
         #[cfg(all(feature = "vanguards", feature = "hs-common"))]
-        let vanguards = VanguardMgr::new(&Default::default(), TestingStateMgr::default()).unwrap();
+        let vanguards = VanguardMgr::new(
+            &Default::default(),
+            MockRuntime::new(),
+            TestingStateMgr::default(),
+            false,
+        )
+        .unwrap();
 
         let (path, usage, _, _) = TargetCircUsage::TimeoutTesting
             .build_path(
