@@ -1,15 +1,6 @@
 //! Support for reporting the status of an onion service.
 
-use std::{
-    sync::{Arc, Mutex},
-    time::SystemTime,
-};
-
-use futures::StreamExt as _;
-use retry_error::RetryError;
-use tor_async_utils::PostageWatchSenderExt;
-
-use crate::{DescUploadError, FatalError, IptError};
+use crate::internal_prelude::*;
 
 /// The current reported status of an onion service.
 #[derive(Debug, Clone, Eq, PartialEq)]
@@ -243,12 +234,12 @@ impl futures::Stream for OnionServiceStatusStream {
 #[derive(Clone)]
 pub(crate) struct StatusSender(Arc<Mutex<postage::watch::Sender<OnionServiceStatus>>>);
 
-/// A handle that can be used by the [`IptManager`](crate::ipt_mgr::IptManager)
+/// A handle that can be used by the [`IptManager`]
 /// to update the [`OnionServiceStatus`].
 #[derive(Clone, derive_more::From)]
 pub(crate) struct IptMgrStatusSender(StatusSender);
 
-/// A handle that can be used by the [`Publisher`](crate::svc::publish::Publisher)
+/// A handle that can be used by the [`Publisher`]
 /// to update the [`OnionServiceStatus`].
 #[derive(Clone, derive_more::From)]
 pub(crate) struct PublisherStatusSender(StatusSender);
@@ -256,7 +247,7 @@ pub(crate) struct PublisherStatusSender(StatusSender);
 /// A helper for implementing [`PublisherStatusSender`] and [`IptMgrStatusSender`].
 ///
 /// TODO: this macro is a bit repetitive, it would be nice if we could reduce duplication even
-/// furhter (and auto-generate a `note_<state>` function for every `State` variant).
+/// further (and auto-generate a `note_<state>` function for every `State` variant).
 macro_rules! impl_status_sender {
     ($sender:ident, $field:ident) => {
         impl $sender {
