@@ -46,12 +46,12 @@
 //! let queue: Arc<TrackingQueue> = account.register_participant_with(
 //!     runtime.now_coarse(),
 //!     |partn| {
-//!         Ok::<_, Void>(Arc::new(TrackingQueue(Mutex::new(Ok(Inner {
+//!         Ok::<_, Void>((Arc::new(TrackingQueue(Mutex::new(Ok(Inner {
 //!             partn,
 //!             data: VecDeque::new(),
-//!         })))))
+//!         })))), ()))
 //!     },
-//! ).unwrap().void_unwrap();
+//! ).unwrap().void_unwrap().0;
 //!
 //! queue.push(runtime.now_coarse(), Box::new([0; 24])).unwrap();
 //! ```
@@ -66,7 +66,7 @@ mod reclaim;
 mod total_qty_notifier;
 
 #[cfg(test)]
-mod test;
+pub(crate) mod test;
 
 use bookkeeping::{BookkeepableQty, ClaimedQty, ParticipQty, TotalQty};
 use total_qty_notifier::TotalQtyNotifier;
@@ -508,6 +508,16 @@ impl MemoryQuotaTracker {
         Ok(tracker)
     }
 
+    /// Returns an estimate of the total memory use
+    ///
+    /// The returned value is:
+    ///
+    ///  * [Approximate.](../index.html#is-approximate)
+    ///  * A snapshot as of the current moment (and there is no way to await changes)
+    pub fn used_current_approx(&self) -> Result<usize, TrackerCorrupted> {
+        Ok(*self.lock()?.total_used.as_raw())
+    }
+
     /// Make a new `Account`
     ///
     /// To actually record memory usage, a Participant must be added.
@@ -680,11 +690,11 @@ impl Account {
     /// in which case `register_participant_with` returns `Ok(Err(E))`.
     ///
     /// On successful setup of the Participant, returns `Ok(Ok(Arc<P>))`.
-    pub fn register_participant_with<P: IsParticipant, E>(
+    pub fn register_participant_with<P: IsParticipant, X, E>(
         &self,
         now: CoarseInstant,
-        constructor: impl FnOnce(Participation) -> Result<Arc<P>, E>,
-    ) -> Result<Result<Arc<P>, E>, Error> {
+        constructor: impl FnOnce(Participation) -> Result<(Arc<P>, X), E>,
+    ) -> Result<Result<(Arc<P>, X), E>, Error> {
         use std::sync::atomic::{AtomicBool, Ordering};
 
         /// Temporary participant, which stands in during constructon
@@ -720,7 +730,7 @@ impl Account {
         // (And, also, we don't want the constructor panicking to poison the whole tracker.)
         // But it means there can be quite a lot of concurrent excitement,
         // including, theoretically, a possible reclaim.
-        let particip = match constructor(partn) {
+        let (particip, xdata) = match constructor(partn) {
             Ok(y) => y,
             Err(e) => return Ok(Err(e)),
         };
@@ -746,7 +756,7 @@ impl Account {
 
         let particip = particip.promise_dropping_is_ok();
         r?;
-        Ok(Ok(particip))
+        Ok(Ok((particip, xdata)))
     }
 
     /// Obtains a handle for the `MemoryQuotaTracker`
@@ -934,6 +944,35 @@ impl Participation {
             tracker: self.tracker.clone(),
         }
     }
+
+    /// Destroy this participant
+    ///
+    /// Treat as freed all the memory allocated via this `Participation` and its clones.
+    /// After this, other clones of this `Participation` are no longer useable:
+    /// attempts to do so will give errors.
+    /// (although they can still be used to get at the `Account`, if it still exists).
+    ///
+    /// The actual memory should be freed promptly.
+    ///
+    /// (It is not necessary to call this function in order to get the memory tracker
+    /// to free its handle onto the `IsParticipant`,
+    /// because the memory quota system holds only a [`Weak`] reference.)
+    pub fn destroy_participant(mut self) {
+        (|| {
+            find_in_tracker! {
+                self.tracker => + tracker, state;
+                self.aid => arecord;
+                ?None
+            };
+            if let Some(mut removed) =
+                refcount::slotmap_remove_early(&mut arecord.ps, self.pid.take())
+            {
+                removed.auto_release(&mut state.global);
+            }
+            Some(())
+        })();
+        // self will be dropped now, but we have already cleared it out.
+    }
 }
 
 impl Clone for Participation {
