@@ -2,6 +2,7 @@
 //! for byte-oriented communication.
 
 use crate::{Error, Result};
+use futures::future::BoxFuture;
 use tor_cell::relaycell::msg::EndReason;
 use tor_cell::relaycell::RelayCmd;
 
@@ -27,7 +28,7 @@ use std::sync::{Mutex, Weak};
 
 use educe::Educe;
 
-#[cfg(feature = "experimental-api")]
+#[cfg(any(feature = "experimental-api", feature = "stream-ctrl"))]
 use crate::circuit::ClientCirc;
 
 use crate::circuit::StreamTarget;
@@ -96,6 +97,19 @@ use super::AnyCmdChecker;
 /// This type is internally composed of a [`DataReader`] and a [`DataWriter`]; the
 /// `DataStream::split` method can be used to split it into those two parts, for more
 /// convenient usage with e.g. stream combinators.
+///
+/// # How long does a stream live?
+///
+/// A `DataStream` will live until all references to it are dropped,
+/// or until it is closed explicitly.
+///
+/// If you split the stream into a `DataReader` and a `DataWriter`, it
+/// will survive until _both_ are dropped, or until it is closed
+/// explicitly.
+///
+/// A stream can also close because of a network error,
+/// or because the other side of the stream decided to close it.
+///
 // # Semver note
 //
 // Note that this type is re-exported as a part of the public API of
@@ -163,6 +177,23 @@ pub struct DataStreamCtrl {
 /// If the `tokio` crate feature is enabled, this type also implements
 /// [`tokio::io::AsyncWrite`](tokio_crate::io::AsyncWrite) for easier integration
 /// with code that expects that trait.
+///
+/// # Drop and close
+///
+/// Note that dropping a `DataWriter` has no special effect on its own:
+/// if the `DataWriter` is dropped, the underlying stream will still remain open
+/// until the `DataReader` is also dropped.
+///
+/// If you want the stream to close earlier, use [`close`](futures::io::AsyncWriteExt::close)
+/// (or [`shutdown`](tokio_crate::io::AsyncWriteExt::shutdown) with `tokio`).
+///
+/// Remember that Tor does not support half-open streams:
+/// If you `close` or `shutdown` a stream,
+/// the other side will not see the stream as half-open,
+/// and so will (probably) not finish sending you any in-progress data.
+/// Do not use `close`/`shutdown` to communicate anything besides
+/// "I am done using this stream."
+///
 // # Semver note
 //
 // Note that this type is re-exported as a part of the public API of
@@ -382,7 +413,7 @@ impl DataStream {
     /// is received to indicate an error.
     ///
     /// Does nothing if this stream is already connected.
-    pub(crate) async fn wait_for_connection(&mut self) -> Result<()> {
+    pub async fn wait_for_connection(&mut self) -> Result<()> {
         // We must put state back before returning
         let state = self.r.state.take().expect("Missing state in DataReader");
 
@@ -541,16 +572,23 @@ impl DataWriter {
         let state = self.state.take().expect("Missing state in DataWriter");
 
         // TODO: this whole function is a bit copy-pasted.
-
-        let mut future = match state {
+        let mut future: BoxFuture<_> = match state {
             DataWriterState::Ready(imp) => {
                 if imp.n_pending == 0 {
                     // Nothing to flush!
-                    self.state = Some(DataWriterState::Ready(imp));
-                    return Poll::Ready(Ok(()));
+                    if should_close {
+                        // We need to actually continue with this function to do the closing.
+                        // Thus, make a future that does nothing and is ready immediately.
+                        Box::pin(futures::future::ready((imp, Ok(()))))
+                    } else {
+                        // There's nothing more to do; we can return.
+                        self.state = Some(DataWriterState::Ready(imp));
+                        return Poll::Ready(Ok(()));
+                    }
+                } else {
+                    // We need to flush the buffer's contents; Make a future for that.
+                    Box::pin(imp.flush_buf())
                 }
-
-                Box::pin(imp.flush_buf())
             }
             DataWriterState::Flushing(fut) => fut,
             DataWriterState::Closed => {
@@ -564,8 +602,15 @@ impl DataWriter {
                 self.state = Some(DataWriterState::Closed);
                 Poll::Ready(Err(e.into()))
             }
-            Poll::Ready((imp, Ok(()))) => {
+            Poll::Ready((mut imp, Ok(()))) => {
                 if should_close {
+                    // Tell the StreamTarget to close, so that the reactor
+                    // realizes that we are done sending. (Dropping `imp.s` does not
+                    // suffice, since there may be other clones of it.  In particular,
+                    // the StreamReader has one, which it uses to keep the stream
+                    // open, among other things.)
+                    imp.s.close();
+
                     #[cfg(feature = "stream-ctrl")]
                     {
                         // TODO RPC:  This is not sufficient to track every case
