@@ -29,10 +29,6 @@ pub trait Object: DowncastSync + Send + Sync + 'static {
     /// stream, it needs to declare what RPC context (like a `TorClient`) it's
     /// using, which requires that some identifier for that context exist
     /// outside of the RPC session that owns it.
-    //
-    // TODO RPC: It would be neat if this were automatically set to true if and
-    // only if there were any "out-of-session psuedomethods" defined on the
-    // object.
     fn expose_outside_of_session(&self) -> bool {
         false
     }
@@ -286,9 +282,7 @@ define_derive_deftly! {
         }}
 
         fn get_cast_table(&self) -> &$crate::CastTable {
-            // TODO RPC: Is there a better way to check "is this a generic type"?
-            // See derive-deftly#37
-            ${if not(approx_equal({$tgens}, {})) {
+            ${if tgens {
                 // For generic types, we have a potentially unbounded number
                 // of CastTables: one for each instantiation of the type.
                 // Therefore we keep a mutable add-only HashMap of CastTables.
@@ -372,12 +366,21 @@ mod test {
         }
     }
 
+    #[derive(Deftly)]
+    #[derive_deftly(Object)]
+    struct Opossum {}
+
     #[test]
     fn standard_cast() {
         let bike = Bicycle {};
         let erased_bike: &dyn Object = &bike;
         let has_wheels: &dyn HasWheels = erased_bike.cast_to_trait().unwrap();
         assert_eq!(has_wheels.num_wheels(), 2);
+
+        let pogo = Opossum {};
+        let erased_pogo: &dyn Object = &pogo;
+        let has_wheels: Option<&dyn HasWheels> = erased_pogo.cast_to_trait();
+        assert!(has_wheels.is_none());
     }
 
     #[derive(Deftly)]
@@ -407,6 +410,9 @@ mod test {
             erased_arc_bytes.clone().cast_to_arc_trait().ok().unwrap();
         assert_eq!(arc_has_wheels.num_wheels(), 4);
 
+        let ref_has_wheels: &dyn HasWheels = erased_arc_bytes.cast_to_trait().unwrap();
+        assert_eq!(ref_has_wheels.num_wheels(), 4);
+
         trait SomethingElse {}
         let arc_something_else: Result<Arc<dyn SomethingElse>, _> =
             erased_arc_bytes.clone().cast_to_arc_trait();
